#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>
#include <string.h>	/* strerror */
#include <time.h>	/* nanosleep */
#include <dirent.h>	/* DIR */
#include <errno.h>
#include <dlfcn.h>

/* prototypes*/
static void err_quit(const char *, ...);
static void err_sys(const char *, ...);

/* global variables */
static struct dirent *(*o_readdir) (DIR *) = NULL;
static unsigned int sleep_every = 3;

static void dlinit(void)
{
    char *error, *p;

    o_readdir = dlsym(RTLD_NEXT, "readdir");
    if ((error = dlerror()) != NULL)
	err_quit("dlsym: %s", error);

    p = getenv("READDIR_SLEEPS_EVERY");
    if (p)
	sleep_every = atoi(p);

    if (!sleep_every)
	err_quit("\nlibreaddir-wrapper.so: $READDIR_SLEEPS_EVERY is invalid\n");
}

struct dirent *readdir(DIR *dir)
{
    static unsigned int counter = 0;
    const struct timespec sleep_for = { 0, 1 };

    if (!o_readdir)
	dlinit();

    if ((counter++ % sleep_every) == 0)
	if (nanosleep(&sleep_for, NULL) < 0 && errno != EINTR)
	    err_sys("\n\nFATAL ERROR!\nnanosleep");

    return (*o_readdir) (dir);
}

/* Error routines */
static void err_sys(const char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    vfprintf(stderr, fmt, ap);
    fprintf(stderr, ": %s\n", strerror(errno));
    va_end(ap);
    exit(1);
}

static void err_quit(const char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    vfprintf(stderr, fmt, ap);
    fputs("\n", stderr);
    va_end(ap);
    exit(1);
}
